import 'dart:developer';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:hub/backend/schema/enums/enums.dart';
import 'package:hub/components/atomic_components/menu_button_item/menu_button_item_widget.dart';
import 'package:hub/components/atomic_components/menu_card_item/menu_card_item.dart';
import 'package:hub/components/molecular_components/menu_item/menu_item.dart';
import 'package:hub/flutter_flow/flutter_flow_util.dart';
import 'package:hub/flutter_flow/nav/nav.dart';
import 'package:hub/shared/extensions/dialog_extensions.dart';
import 'package:hub/shared/helpers/license/license_helper.dart';

class MenuEvent {}

class MenuState {
  final List<MenuEntry?> menuEntries;
  final bool isGrid;

  MenuState({this.menuEntries = const [], this.isGrid = false});

  MenuState copyWith({List<MenuEntry?>? menuEntries, bool? isGrid}) {
    return MenuState(
      menuEntries: menuEntries ?? this.menuEntries,
      isGrid: isGrid ?? this.isGrid,
    );
  }
}

class MenuBloc extends Bloc<MenuEvent, MenuState> {
  final MenuView style;
  final MenuItem item;
  final bool expandable;
  final List<Module> menuOptions;

  MenuBloc({required this.style, required this.item, required this.expandable, required this.menuOptions}) : super(MenuState()) {
    on<MenuEvent>((event, emit) async {
      final entries = await generateMenuEntries();
      emit(state.copyWith(menuEntries: entries));
    });
  }

  Future<List<MenuEntry?>> generateMenuEntries() async {
    List<MenuEntry?> entries = [];
    Future<MenuEntry?> addMenuEntry(IconData icon, String text, Function() action) async {
      entries.add(
        item == MenuItem.button
            ? MenuButtonWidget(icon: icon, action: action, title: text)
            : item == MenuItem.card
                ? MenuCardItem(icon: icon, action: action, title: text)
                : item == MenuItem.tile
                    ? MenuCardItem(icon: icon, action: action, title: text)
                    : null,
      );
      return null;
    }

    for (var opt in menuOptions) {
      String? v = await LicenseHelper().g(opt.value);
      log('Module: ${opt.value} - License: $v');
      switch (v) {
        case 'VISIVEL':
          await addMenuEntry(opt.icon, opt.name, () async {
            await nav(opt.route);
          });
          continue;
        case 'DESABILITADO':
          await addMenuEntry(opt.icon, opt.name, () async {
            await DialogUnavailable.unavailableFeature(key.currentContext!);
          });
          continue;
        case 'INVISIVEL':
          continue;
        default:
          continue;
      }
    }

    return entries;
  }

  Future nav(String link) async {
    log('Opening: $link');
    key.currentContext!.push(link, extra: <String, dynamic>{
      kTransitionInfoKey: const TransitionInfo(
        hasTransition: false,
        transitionType: PageTransitionType.scale,
        alignment: Alignment.bottomCenter,
      ),
    });
  }

  Future<void> logout() async {
    // Implement logout logic here
  }

  Future settings() async {
    // Implement settings navigation logic here
  }

  Future about() async {
    // Implement about navigation logic here
  }
}
